import SimpleHTTPServer
import SocketServer
from optparse import OptionParser
import webbrowser
import os.path
import time
import json


parser = OptionParser()
parser.add_option("-p","--port", dest="port", default = 8003, help = "Port to listen on.")
parser.add_option("-d","--docroot", dest="docroot", default = "", help = "A path representing the docroot. If present in a request, it will be stripped.")
parser.add_option("-w","--waittime", dest="wait", default=0, help = "A universal time to wait before responding to each request.")
(options, args) = parser.parse_args()

PORT = int(options.port)
PREFIX = options.docroot
sleep_queue = []
wait_time = 0
if options.wait:
   wait_time = int(options.wait)

class SimpleHTTPRequestPrefixableHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):

    def do_GET(self):
        filename = os.path.basename(self.path)
        path = os.path.dirname(self.path)
        if filename.find('@') >= 0:
            elements = filename.split('@')
            filename = elements[0]
            waitsecs = int(elements[1])
            time.sleep(waitsecs)
        elif sleep_queue:
            time_to_sleep = sleep_queue.pop(0)
            print "Sleeping for %d seconds!" % time_to_sleep 
            time.sleep(time_to_sleep)
            print "AWAKE!"
        elif wait_time:
            print "Sleeping for %d seconds!" % wait_time 
            time.sleep(wait_time)
            print "AWAKE!"

        if len(PREFIX):
            path = self.path.replace(PREFIX, "")
        if path != '/':
            self.path = path + "/" + filename
        else:
            self.path = path + filename
        SimpleHTTPServer.SimpleHTTPRequestHandler.do_GET(self)
    
    def send_head(self):
       """Common code for GET and HEAD commands.
      
       This sends the response code and MIME headers.
      
       Return value is either a file object (which has to be copied
       to the outputfile by the caller unless the command was HEAD,
       and must be closed by the caller under all circumstances), or
       None, in which case the caller has nothing further to do.
      
       """
       path = self.translate_path(self.path)
       f = None
       if os.path.isdir(path):
           if not self.path.endswith('/'):
               # redirect browser - doing basically what apache does
               self.send_response(301)
               self.send_header("Location", self.path + "/")
               self.end_headers()
               return None
           for index in "index.html", "index.htm":
               index = os.path.join(path, index)
               if os.path.exists(index):
                   path = index
                   break
           else:
               return self.list_directory(path)
       ctype = self.guess_type(path)
       try:
           # Always read in binary mode. Opening files in text mode may cause
           # newline translations, making the actual size of the content
           # transmitted *less* than the content-length!
           f = open(path, 'rb')
       except IOError:
           self.send_error(404, "File not found")
           return None
       self.send_response(200)
       self.send_header("Content-type", ctype)
       fs = os.fstat(f.fileno())
       self.send_header("Content-Length", str(fs[6]))
       self.send_header("no-cache", "true")
       self.send_header("Last-Modified", self.date_time_string(fs.st_mtime))
       self.send_header("Pragma","no-cache")
       self.send_header("Expires", "Thu, 1 Jan 1970 00:00:00 GMT")
       self.send_header("Cache-Control", "max-age=0, no-store, no-cache, must-revalidate")
       self.end_headers()
       return f


    def do_POST(self):
        d = self.rfile.readline()
        s = json.loads(d)
        if s.has_key("verb") and s['verb'] == "clear":
            print "Clearing queue"
            while sleep_queue:
                sleep_queue.pop()
        elif s.has_key("verb") and s['verb'] == "set":
            print "Setting sleep queue : %s" % s['queue']
            sleep_queue.extend(s['queue'])

Handler = SimpleHTTPRequestPrefixableHandler
httpd = SocketServer.TCPServer(("", PORT), Handler)
print "serving at port", PORT
httpd.serve_forever(0.5)
